@@ -1,6 +1,5 @@
 """Main module."""
 import asyncio
-import json
 import logging
 from datetime import datetime, timedelta, timezone
 from http import HTTPStatus
@@ -97,6 +96,17 @@ def _timestep_to_key(timestep: str) -> str:
     return NOWCAST
 
 
+def mask(text: str) -> str:
+    """Mask 3/4 of a string."""
+    text_len = len(text)
+    masked_char_len = text_len * 3 // 4
+    unmasked_char_len = text_len - masked_char_len
+    prefix = text[0 : (unmasked_char_len // 2)]
+    suffix_len = len(prefix) + (1 if unmasked_char_len % 2 else 0)
+    suffix = text[-(suffix_len):]
+    return f"{prefix}{'*' * masked_char_len}{suffix}"
+
+
 class TomorrowioV4:
     """Async class to query the Tomorrow.io v4 API."""
 
@@ -151,6 +161,11 @@ class TomorrowioV4:
         """The number of API requests made during the most recent call."""
         return self._num_api_requests
 
+    @property
+    def api_key_masked(self) -> str:
+        """Return the API key with the first 3/4 masked."""
+        return mask(self.api_key)
+
     @staticmethod
     def convert_fields_to_measurements(fields: List[str]) -> List[str]:
         """Converts general field list into fields with measurements."""
@@ -182,32 +197,52 @@ class TomorrowioV4:
 
     @staticmethod
     def _get_url() -> str:
+        """Return base URL for API requests."""
         # This method is required for test mocks
         return BASE_URL_V4
 
     async def _call_api(self, params: Dict[str, Any]) -> Dict[str, Any]:
+        """Call tomorrow.io API."""
         if self._session:
-            return await self._make_call(params, self._session)
+            return await self.__call_api(params, self._session)
 
         async with ClientSession() as session:
-            return await self._make_call(params, session)
+            return await self.__call_api(params, session)
 
-    async def _make_call(
+    async def __call_api(
         self, params: Dict[str, Any], session: ClientSession
     ) -> Dict[str, Any]:
+        """Make API call with session."""
         if self._remaining_requests_in_second == 0:
             await asyncio.sleep(1)
 
+        payload = {**self._params, **params}
+
+        _LOGGER.debug(
+            "Sending the following payload to tomorrow.io: %s",
+            {
+                **payload,
+                "location": mask(payload["location"]),
+            },
+        )
+
         try:
             resp = await session.post(
                 self._get_url(),
                 headers=self._headers,
-                data=json.dumps({**self._params, **params}),
+                json=payload,
+                compress=False,
             )
-            resp_json = await resp.json()
+            resp_json = await resp.json(content_type=None)
         except ClientConnectionError as error:
             raise CantConnectException() from error
 
+        _LOGGER.debug(
+            "Received a response with status code %s and headers %s",
+            resp.status,
+            resp.headers,
+        )
+
         self._rate_limits = CIMultiDict(
             {k: int(v) for k, v in resp.headers.items() if "ratelimit" in k.lower()}
         )
